/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.skywalking.generator;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import lombok.Data;

@JsonDeserialize(builder = FixedStringGenerator.Builder.class)
public final class FixedStringGenerator implements Generator<String> {
    private final String value;

    public FixedStringGenerator(Builder builder) {
        value = builder.value;
    }

    @Override
    public String next() {
        return value;
    }

    @Data
    public static class Builder {
        private String value;

        public FixedStringGenerator build() {
            return new FixedStringGenerator(this);
        }
    }

    @Override
    public String toString() {
        return String.valueOf(next());
    }
}
